@@ -229,6 +229,10 @@ def BER(X ,y, M = 10000, NSigma = 10, show = False, save_fig = ''):
     return BER, IMG #, BER2
 
 def Mean_KLD (P,Q):
+    '''
+    Calculate the mean KL divergence between ground truth and predicted one-hot encodings for an entire data set.
+    P and Q must be both m x K numpy arrays. m = sample number, K = class number
+    '''
     klds = []
     for idx, ground_truth in enumerate(P):
         prediction = Q[idx]
@@ -620,7 +624,11 @@ def MANOVA(X,y, verbose = False):
     X2 = X[:,1]
     df = pd.DataFrame({'X1': X1,'X2': X2,'y':y})
     mv = manova.MANOVA.from_formula('X1 + X2 ~ y', data=df) # Intercept is included by default.
-    r = mv.mv_test()       
+    
+    try:
+        r = mv.mv_test() 
+    except: # LinAlgError: Singular matrix     
+        return math.nan, math.nan, 'Exception in MANOVA'
     
     LOG = ''
     LOG += 'endog: ' + str(mv.endog_names) + '\n'
@@ -1008,10 +1016,13 @@ def get_metrics(X,y):
     dct['test.ANOVA.F'] = F
     dct['test.ANOVA.F.max'] = np.max(F)
 
-    p, F, _ = MANOVA(X,y)
-    dct['test.MANOVA'] = p
-    dct['test.MANOVA.log10'] = np.log10 (p)
-    dct['test.MANOVA.F'] = F
+    p, F, log = MANOVA(X,y)
+    if log == 'Exception in MANOVA':
+        pass
+    else:
+        dct['test.MANOVA'] = p
+        dct['test.MANOVA.log10'] = np.log10 (p)
+        dct['test.MANOVA.F'] = F
 
     p, U, _ = MWW(X,y)
     dct['test.MWW'] = p
@@ -1087,12 +1098,15 @@ def get_html(X,y):
     anova_p, _, anova_img = ANOVA(X,y)
     
     tr = '<tr><td> ANOVA p' + str(anova_p) + '<br/>' + anova_img + '</td><tr>'
-    html += tr    
-    
+    html += tr        
+
     manova_p, _, manova_log = MANOVA(X,y)
-    
-    tr = '<tr><td> MANOVA p = ' + str(manova_p) + '<br/><pre>' + manova_log + '</pre></td><tr>'
-    html += tr  
+
+    if manova_log == 'Exception in MANOVA':
+        pass
+    else:
+        tr = '<tr><td> MANOVA p = ' + str(manova_p) + '<br/><pre>' + manova_log + '</pre></td><tr>'
+        html += tr  
     
     mww_p, _, mww_img = MWW(X,y)
     
